# /* Copyright 2002-2023 CS GROUP
#  * Licensed to CS GROUP (CS) under one or more
#  * contributor license agreements.  See the NOTICE file distributed with
#  * this work for additional information regarding copyright ownership.
#  * CS licenses this file to You under the Apache License, Version 2.0
#  * (the "License"); you may not use this file except in compliance with
#  * the License.  You may obtain a copy of the License at
#  *
#  *   http://www.apache.org/licenses/LICENSE-2.0
#  *
#  * Unless required by applicable law or agreed to in writing, software
#  * distributed under the License is distributed on an "AS IS" BASIS,
#  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  * See the License for the specific language governing permissions and
#  * limitations under the License.
#  */
# package org.orekit.propagation.analytical;
#
# import org.hipparchus.ode.events.Action;
# import org.hipparchus.util.FastMath;
# import org.junit.jupiter.api.Assertions;
# import org.junit.jupiter.api.BeforeEach;
# import org.junit.jupiter.api.Test;
# import org.orekit.Utils;
# import org.orekit.bodies.CelestialBodyFactory;
# import org.orekit.bodies.OneAxisEllipsoid;
# import org.orekit.errors.OrekitException;
# import org.orekit.frames.Frame;
# import org.orekit.frames.FramesFactory;
# import org.orekit.orbits.KeplerianOrbit;
# import org.orekit.orbits.Orbit;
# import org.orekit.orbits.OrbitType;
# import org.orekit.orbits.PositionAngleType;
# import org.orekit.propagation.BoundedPropagator;
# import org.orekit.propagation.Propagator;
# import org.orekit.propagation.SpacecraftState;
# import org.orekit.propagation.SpacecraftStateInterpolator;
# import org.orekit.propagation.events.EclipseDetector;
# import org.orekit.propagation.events.EventDetector;
# import org.orekit.propagation.events.handlers.EventHandler;
# import org.orekit.time.AbsoluteDate;
# import org.orekit.time.DateComponents;
# import org.orekit.time.TimeComponents;
# import org.orekit.time.TimeInterpolator;
# import org.orekit.time.TimeScalesFactory;
# import org.orekit.utils.IERSConventions;
#
# import java.util.ArrayList;
# import java.util.List;
import orekit
from org.orekit.propagation.analytical import Ephemeris, EcksteinHechlerPropagator

orekit.initVM()
import unittest


from org.hipparchus.ode.events import Action
from org.hipparchus.util import FastMath
from org.orekit.bodies import CelestialBodyFactory, OneAxisEllipsoid
from org.orekit.errors import OrekitException
from org.orekit.frames import Frame, FramesFactory
from org.orekit.orbits import KeplerianOrbit, Orbit, OrbitType, PositionAngleType
from org.orekit.propagation import BoundedPropagator, Propagator, SpacecraftState, SpacecraftStateInterpolator
from org.orekit.propagation.events import EclipseDetector, EventDetector
from org.orekit.propagation.events.handlers import EventHandler, PythonEventHandler
from org.orekit.time import AbsoluteDate, DateComponents, TimeComponents, TimeScalesFactory
from org.orekit.utils import IERSConventions
from java.util import ArrayList, List


from orekit.pyhelpers import setup_orekit_curdir

setup_orekit_curdir("resources")


class MyEventHandler(PythonEventHandler):
    
    orb_type = None
    mainclass = None
    
    def __init__(self, orb_type:OrbitType, mainclass):
        self.orb_type = orb_type
        self.mainclass = mainclass
        super(MyEventHandler, self).__init__()
    

    def init(self, *args):
        self.mainclass.inEclipsecounter = 0
        self.mainclass.outEclipsecounter = 0

    def eventOccurred(self, s: SpacecraftState, eventDetector: EventDetector, increasing: bool) -> Action:
        assert self.orb_type == s.getOrbit().getType()
        if increasing:
            mainclass.inEclipsecounter += 1
        else:
            mainclass.outEclipsecounter += 1
        return Action.CONTINUE

    def resetState(self, eventDetector, spacecraftState):
        pass

class EphemerisEventsTest(unittest.TestCase):

    def testEphemKeplerian(self):
        self.checkEphem(OrbitType.KEPLERIAN)

    def testEphemCircular(self):
        self.checkEphem(OrbitType.CIRCULAR)
   
    def testEphemEquinoctial(self):
        self.checkEphem(OrbitType.EQUINOCTIAL)
            
    def testEphemCartesian(self):
        self.checkEphem(OrbitType.CARTESIAN)
    
    def buildEphem(self, orb_type: OrbitType):
        mass = 2500.0
        a = 7187990.1979844316
        e = 0.5e-4
        i = 1.7105407051081795
        omega = 1.9674147913622104
        OMEGA = FastMath.toRadians(261.0)
        lv = 0.0
        mu  = 3.9860047e14
        ae  = 6.378137e6
        c20 = -1.08263e-3
        c30 = 2.54e-6
        c40 = 1.62e-6
        c50 = 2.3e-7
        c60 = -5.5e-7

        deltaT = self.finalDate.durationFrom(self.initDate)

        frame = FramesFactory.getEME2000()

        transPar = KeplerianOrbit(a, e, i, omega, OMEGA, lv, PositionAngleType.TRUE, frame, self.initDate, mu)

        nbIntervals = 720
        propagator = EcksteinHechlerPropagator(transPar, mass, ae, mu, c20, c30, c40, c50, c60)

        tab = ArrayList(nbIntervals + 1)
        for j in range(nbIntervals + 1):
            state = propagator.propagate(self.initDate.shiftedBy((j * deltaT) / nbIntervals))
            tab.add(SpacecraftState(orb_type.convertType(state.getOrbit()), state.getAttitude(), state.getMass()))

        interpolator = SpacecraftStateInterpolator(2, frame, frame)

        return Ephemeris(tab, interpolator)

    def buildEclipseDetector(self, orb_type: OrbitType):
        sunRadius = 696000000.
        earthRadius = 6400000.

        ecl = EclipseDetector(CelestialBodyFactory.getSun(), sunRadius,
                              OneAxisEllipsoid(earthRadius,
                                               0.0,
                                               FramesFactory.getITRF(IERSConventions.IERS_2010, True)))
        ecl = ecl.withMaxCheck(60.0).withThreshold(1.0e-3).withHandler(MyEventHandler(orb_type, self))
        return ecl


    def checkEphem(self, orb_type:OrbitType):
        self.initDate = AbsoluteDate(DateComponents(2004, 1, 1),
                                        TimeComponents.H00,
                                        TimeScalesFactory.getUTC())


        self.finalDate = AbsoluteDate(DateComponents(2004, 1, 2),
                            TimeComponents.H00,
                                        TimeScalesFactory.getUTC())

        ephem = self.buildEphem(orb_type)

        ephem.addEventDetector(self.buildEclipseDetector(orb_type))

        computeEnd = AbsoluteDate(self.finalDate, -1000.0)

        Propagator.cast_(ephem).clearStepHandlers()
        state = Propagator.cast_(ephem).propagate(computeEnd)
        handler = ephem.getEventsDetectors().iterator().next().getHandler()
        self.assertEqual(computeEnd, state.getDate())
        self.assertEqual(14, self.inEclipsecounter)
        self.assertEquals(14, self.outEclipsecounter)

    def setUp(self):
        pass
